﻿public class Program {
    static void Main(string[] args) {
        var exitCommand = "nothing";
        string backupLocation = "/home/rene/";
        var pacmanWrapper = "yay";
        var backupAmount = 2;

        if (args.Length == 0) {
            beforeUpdate();
            Console.WriteLine(pacmanWrapper);
            afterUpdate();
            return;
        }

        string arg = args[0].TrimStart('-');;
        if (args[0].StartsWith("--")) {
            switch (arg) {
                case "help":
                    Console.WriteLine(help());
                    break;
                case "preview":
                    // need to execute 'yay -Sy && yay -Qu'
                    break;
                case "version":
                    Console.WriteLine("version");
                    break;
                case "backup":
                    beforeUpdate();
                    break;
                default:
                    return;
            }
            return;
        }

        beforeUpdate();

        foreach (var option in arg) {
            switch (option) {
                case 'f':
                    // flatpak update -u --noninteractive && flatpak list > "$TMP"/after-backup_"$DATE"/flatpak-after.txt
                    Console.WriteLine("flatpak update");
                    break;
                case 'p':
                    // "$PACMAN_WRAPPER" && pacman -Qq > "$TMP"/after-backup_"$DATE"/pacman-after.txt
                    Console.WriteLine(pacmanWrapper);
                    break;
                case 'a':
                    // 'p' && 'f'
                    Console.WriteLine("flatpak update && " + pacmanWrapper);
                    break;
                case 'g':
                    if (args[0].TrimStart('-') == "g") {
                        Console.WriteLine(pacmanWrapper);
                    }
                    exitCommand = "systemctl -i poweroff";
                    break;
                case 'r':
                    if (args[0].TrimStart('-') == "r") {
                        Console.WriteLine(pacmanWrapper);
                    }
                    exitCommand = "systemctl -i reboot";
                    break;
                default:
                    help();
                    return;
            }
        }

        afterUpdate();

        void readConfig() {
            backupLocation = "/mnt/Hephaistos/update/";
            pacmanWrapper = "yay";
            backupAmount = 2;

            if (!Directory.Exists(backupLocation)) {
                Console.WriteLine("update-csharp: Backup location does not exist or cannot be reached");
                return;
            }
        }

        //find the amount of backups in the backupLocation and delete the oldest
        void deleteOldestBackup() {
            Console.WriteLine(backupAmount);
        }

        void beforeUpdate() {
            readConfig();
            deleteOldestBackup();

            Update.backup("before");
            Update.zipPacmanDatabase();
            Update.copyToBackupLocation(backupLocation);

            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("pre-backup complete");
            Console.ResetColor();
        }

        void afterUpdate() {
            Update.backup("after");
            Update.copyToBackupLocation(backupLocation);
            Directory.Delete("/tmp/update", true);

            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("after-backup complete");
            Console.ResetColor();
            Console.WriteLine(exitCommand);     // execute exitCommand
        }

        string help() {
            string helpMessage = "Usage: update [OPTION]\n";
            helpMessage += "\noptions:\n";
            helpMessage += "no flag        same as -p\n";
            helpMessage += "-f             updates using flatpak update only\n";
            helpMessage += "-p             updates using a pacman-wrapper only\n";
            helpMessage += "-a             updates using flatpak update and a pacman-wrapper\n";
            helpMessage += "-g             shutdowns the computer afterwards\n";
            helpMessage += "-r             reboots the computer afterwards\n";
            helpMessage += "--help         displays this message\n";
            helpMessage += "--preview      shows a preview of which pkg's can be updated\n";
            helpMessage += "--version      prints out the version number\n";
            helpMessage += "--backup       just does the pre-backup without updating";
            return helpMessage;
        }
    }
}