using System.Diagnostics;
using System.IO.Compression;

public class Update {
    static string tmpDirectory = "/tmp/update/";
    public static void backup(string mode) {
        string backupDir;
        string pkgList;
        string flatpakList;
        List<string> filesToCopy = new List<string>();
        string zipName;

        switch (mode) {
            case "before":
                backupDir = tmpDirectory + "beforeBackup";
                pkgList = backupDir + "pacman-pre.txt";
                flatpakList = backupDir + "flatpak-pre.txt";
                filesToCopy.Add("/etc/fstab");
                filesToCopy.Add("/etc/makepkg.conf");
                zipName =  tmpDirectory + "beforeBackup.zip";
                break;
            case "after":
                backupDir = tmpDirectory + "afterBackup";
                pkgList = backupDir + "pacman-after.txt";
                flatpakList = backupDir + "flatpak-after.txt";
                zipName = tmpDirectory + "afterBackup.zip";
                break;
            default:
                throw new ArgumentException("No valid mode was given. Valid modes are before and after");
        }

        if (Directory.Exists(tmpDirectory)) {
            Directory.Delete(tmpDirectory, true);
        }
        Directory.CreateDirectory(backupDir);

        foreach (var file in filesToCopy) {
            FileInfo info = new FileInfo(file);
            string destFile = Path.Combine(backupDir, info.Name);
            File.Copy(info.FullName, destFile);
        }

        ZipFile.CreateFromDirectory(backupDir, zipName);
        Directory.Delete(backupDir, true);
    }

    public static void copyToBackupLocation(string backupLocation) {
        foreach (var zip in Directory.GetFiles(tmpDirectory)) {
            FileInfo info = new FileInfo(zip);
            string destFile = Path.Combine(backupLocation, info.Name);
            File.Copy(info.FullName, destFile, true);
        }
    }

    public static void zipPacmanDatabase() {
        string pacmanDatabaseLocation = "/var/lib/pacman/local";
        string zipName = tmpDirectory + "pacmanDatabase.zip";

        if(dbLockExists()) {
            throw new ApplicationException("db.lck exists. Please try again later.");
        }
        
        try {
            lockDatabase();
        
            ZipFile.CreateFromDirectory(pacmanDatabaseLocation, zipName);
        } catch (Exception e) {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine(e.Message);
        } finally {
            unlockDatabase();
        }
    }

    private static bool dbLockExists() {
        if (File.Exists("/var/lib/pacman/db.lck")) {
            return true;
        }
        return false;
    }

    private static void lockDatabase() {
        var psi = new ProcessStartInfo
            {
                FileName = "/bin/bash",
                UseShellExecute = false,
                RedirectStandardOutput = true,
                Arguments = string.Format("-c \"cd /var/lib/pacman/ && sudo touch db.lck && sudo chmod 000 db.lck\"")
            };

        using (var p = Process.Start(psi))
        {
            if (p != null)
            {
                var strOutput = p.StandardOutput.ReadToEnd();
                p.WaitForExit();
            }
        }
    }

    private static void unlockDatabase() {
        var psi = new ProcessStartInfo {
            FileName = "/bin/bash",
            UseShellExecute = false,
            RedirectStandardOutput = true,
            Arguments = string.Format("-c \"cd /var/lib/pacman/ && sudo rm -f db.lck\"")
        };
        using (var p = Process.Start(psi)) {
            if (p != null) {
                var strOutput = p.StandardOutput.ReadToEnd();
                p.WaitForExit();
            }
        }
    }
}